﻿package meyermike
{
    import adobe.utils.CustomActions;
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Shape;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.KeyboardEvent;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.minild9.Music;
    import meyermike.minild9.Player;
    import meyermike.minild10.scripts.Intro;
    import meyermike.script.Script;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Main extends Sprite
    {
        private var mScript:Script;
        private var mNextScript:Script;
        public static var sInstance:Main;
        private var mIsPaused:Boolean;
        private static const CAMERA_TIGHTNESS:Number = 0.125;

        public var mCameraFollowTarget:GameObject;

        // main will set it to true, but it expects scripts to set it false later
        public static var sLevelOver:Boolean;

        public static var mGameHolder:DisplayObjectContainer;

        public function Main():void
        {
            if (stage)
            {
                Init();
            }
            else
            {
                addEventListener(Event.ADDED_TO_STAGE, Init);
            }
        }

        private function Init(e:Event = null):void
        {
            Main.sLevelOver = false;
            Main.sInstance = this;
            mCameraFollowTarget = null;

            stage.frameRate = 30;
            removeEventListener(Event.ADDED_TO_STAGE, Init);

            mIsPaused = false;

            InputUtil.Init();

            addEventListener(Event.ENTER_FRAME, HandleEnterFrame);

            stage.addEventListener(KeyboardEvent.KEY_DOWN, HandleKeyDown);

            SetupScene();

            mNextScript = new Intro();

            stage.addEventListener(Event.DEACTIVATE, Pause);
            stage.addEventListener(Event.ACTIVATE, Unpause);
        }

        /**
         * add the persistent display objects that stick around even between levels
         */
        private function SetupScene():void
        {
            // holder for the game itself (gets moved to scroll the camera)
            mGameHolder = new Sprite();
            this.addChild(mGameHolder);

            // call it once so that the layers know what the container is
            Layers.CreateLayers(mGameHolder);
        }

        private function HandleEnterFrame(e:Event = null):void
        {
            Time.TimeDelta = (1.0 / stage.frameRate);

            ScriptChange();

            if (!mIsPaused)
            {
                Update();
                Integrate();
                PreDraw();
            }
        }

        private function Update():void
        {
            mScript.Update();
            Layers.Update();
            InputUtil.Update();
        }

        private function Integrate():void
        {
            Layers.Integrate();
        }
        private function PreDraw():void
        {
            // scrolling
            if (!Main.sLevelOver)
            {
                if (null != mCameraFollowTarget)
                {
                    if (null != mCameraFollowTarget.parent)
                    {
                        // note: does not account for rotation
                        var actualLoc:Point = new Point(
                            mGameHolder.scaleX * mCameraFollowTarget.x,
                            mGameHolder.scaleY * mCameraFollowTarget.y );

                        var targetx:Number = (0.5 * stage.stageWidth)  - actualLoc.x;
                        var targety:Number = (0.5 * stage.stageHeight) - actualLoc.y;

                        mGameHolder.x += mGameHolder.scaleX * Main.CAMERA_TIGHTNESS * (targetx - mGameHolder.x);
                        mGameHolder.y += mGameHolder.scaleY * Main.CAMERA_TIGHTNESS * (targety - mGameHolder.y);
                    }
                }
            }

            Layers.PreDraw();
        }

        /// Called on each Beat of the game.
        private function Beat(e:Event = null):void
        {
            var beat:Number = mScript.GetCurBeat();

            Layers.Beat(beat);
        }

        private function OffBeat(e:Event = null):void
        {
            var beat:Number = mScript.GetCurBeat();

            Layers.OffBeat(beat);
        }
        private function QuarterBeat(e:Event = null):void
        {
            var beat:Number = mScript.GetCurQuarterBeat();

            Layers.QuarterBeat(beat);
        }

        public function Pause(e:Event = null):void
        {
            mIsPaused = true;
        }
        public function Unpause(e:Event = null):void
        {
            mIsPaused = false;
        }

        /**
         * if a script change was queued up, perform the actual switch now
         */
        private function ScriptChange():void
        {
            if (mNextScript != null)
            {
                EndScript(mScript);
                BeginScript(mNextScript);

                mScript = mNextScript;
                mNextScript = null;
            }
        }

        /**
         * shut it down
         */
        private function EndScript(script:Script):void
        {
            if (script != null)
            {
                script.removeEventListener(Script.EVENT_BEAT, Beat);
                script.removeEventListener(Script.EVENT_OFFBEAT, OffBeat);
                script.removeEventListener(Script.EVENT_QUARTERBEAT, QuarterBeat);
                script.End();
            }
        }

        /**
         *
         */
        private function BeginScript(script:Script):void
        {
            if (script != null)
            {
                script.addEventListener(Script.EVENT_BEAT, Beat);
                script.addEventListener(Script.EVENT_OFFBEAT, OffBeat);
                script.addEventListener(Script.EVENT_QUARTERBEAT, QuarterBeat);
                script.Begin();
            }
        }

        /**
         * access to the script
         */
        public function GetScript():Script
        {
            return mScript;
        }

        /**
         * set a script to switch to once the current beat/update is finished
         */
        public function SwitchToScript(nextScript:Script):void
        {
            mNextScript = nextScript;
        }

        /**
         * Cheat codes!
         */
        private var mCheatString:String = "";
        private static const KEYLOG_MAX_LENGTH:uint = 16;
        private function HandleKeyDown(e:KeyboardEvent):void
        {
            if (0 != e.charCode)
            {
                mCheatString += String.fromCharCode(e.charCode);

                if (mCheatString.length > KEYLOG_MAX_LENGTH)
                {
                    mCheatString = mCheatString.substring(1);
                }

                // next track cheat
                if (0 == mCheatString.indexOf("next"))
                {
                    //Music.StartNewMusic();
                    mCheatString = "";
                }

                // restart game cheat
                if (-1 != mCheatString.indexOf("restart"))
                {
                    // do something here
                    ResetScrolling();
                    SwitchToScript(new Intro());
                    mCheatString = "";
                }
            }
        }

        /**
         * go back to the starting position
         */
        public function ResetScrolling():void
        {
            mGameHolder.x = 0;
            mGameHolder.y = 0;
        }

        /**
         * give it a target to scroll towards
         */
        public function  SetCameraFollowTarget(go:GameObject):void
        {
            mCameraFollowTarget = go;
        }

        /**
         * Allow access to the "camera" position, so that objects can follow the camera.
         * NOTE this actually tells you where the upper-left corner is in the scene
         */
        public function GetCameraPosition():Point
        {
            return new Point( -mGameHolder.x, -mGameHolder.y);
        }

        /**
         *
         */
        public function SetZoom(zoom:Number):void
        {
            mGameHolder.scaleX = zoom;
            mGameHolder.scaleY = zoom;
        }
    }
}
