﻿package meyermike
{
    import flash.events.Event;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.GameObject;
    import meyermike.Layers;
    import meyermike.Main;
    import meyermike.Time;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PlatformerCharacter extends GameObject
    {
        public static const MAX_FALL_SPEED:Number = 600;
        public static var EVENT_DEATH:String = "death";

        private var mDying:Boolean;

        private var mIsOnGround:Boolean;
        public function get IsOnGround():Boolean { return mIsOnGround; }

        /**
         * set initial values after spawning
         */
        protected function Restart():void
        {
            mIsOnGround = false;
            mDying = false;
            this.dr = 0;
            this.rotation = 0;
        }

        /**
         * check whether we're doing the death animation
         */
        public function IsDying():Boolean
        {
            return mDying;
        }

        /**
         * Start doing the dying animation
         */
        public function StartDeath():void
        {
            mDying = true;
            dispatchEvent(new Event(EVENT_DEATH));
        }

        /**
         * Setup that needs to happen before we do the collision checks
         * During collision checks, CollisionOccurred can get called many times.
         * Afterward, we go to PostCollisionCheck to make the actual movement happen.
         */
        public function PreCollisionCheck():void
        {
            mIsOnGround = false;
        }

        /**
         * override to deal with collisions
         */
        public function CollisionOccurred(ci:CollisionInfo, object:Object):void
        {
            var collisionBox:Rectangle = this.GetCollisionBounds();

            this.x = ci.location.x;
            if ( 0.0 < ci.normal.x )
            {
                this.x += (0.5 * collisionBox.width);
            }
            else if ( 0.0 > ci.normal.x )
            {
                this.x -= (0.5 * collisionBox.width);
            }

            this.y = ci.location.y;
            if ( 0.0 < ci.normal.y )
            {
                this.y += (0.5 * collisionBox.height);
            }
            else if ( 0.0 > ci.normal.y )
            {
                this.y -= (0.5 * collisionBox.height);
            }

            if (ci.normal.x != 0.0)
            {
                this.dx = 0.0;
            }
            if (ci.normal.y != 0.0)
            {
                this.dy = 0.0;
            }

            mIsOnGround = mIsOnGround || (ci.normal.y < -Math.SQRT1_2);
        }

        /**
         *
         */
        public function PostCollisionCheck():void
        {
            super.Integrate(Time.TimeDelta);
        }

        /**
         * platformer characters need to do this in postcollision instead
         * reason:  input happens between collisions and this
         */
        public override function Integrate(elapsedTime:Number):void
        {
        }
    }
}
