﻿package meyermike.minild10
{
    import flash.display.Bitmap;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.geom.ColorTransform;
    import flash.geom.Rectangle;
    import flash.ui.Keyboard;
    import meyermike.InputUtil;
    import meyermike.Main;
    import meyermike.minild10.scripts.Intro;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Player extends MiniLD10Character
    {
        // keys you can use to jump
        private static const JUMP_KEYS:Array = [Keyboard.UP, Keyboard.SPACE, Keyboard.SHIFT, 90];

        // keys you can use to punch
        private static const PUNCH_KEYS:Array = [Keyboard.DOWN, Keyboard.CONTROL, 88];

        private var mIsJumping:Boolean = false;
        private var mAirJumpsRemaining:Number = Constants.NUM_AIR_JUMPS;

        // key they used to jump, so we know which key's release event we care about
        private var mJumpKey:uint = 0;

        public override function Init():void
        {
            super.Init();

            CreateGraphics(Constants.PLAYER_COLOR);
            Restart();
        }

        /**
         * determine which animation should be playing
         */
        protected override function Punching(elapsedTime:Number):void
        {
            super.Punching(elapsedTime);

            if (!IsPunching)
            {
                for each (var keyCode:uint in PUNCH_KEYS)
                {
                    if (InputUtil.WasJustPressed(keyCode))
                    {
                        if (this.scaleX < 0)
                        {
                            if (this.dx > -Constants.PUNCH_MINSPEED)
                            {
                                this.dx = -Constants.PUNCH_MINSPEED;
                            }
                        }
                        if (this.scaleX > 0)
                        {
                            if (this.dx < Constants.PUNCH_MINSPEED)
                            {
                                this.dx = Constants.PUNCH_MINSPEED;
                            }
                        }

                        StartPunch();
                        break;
                    }
                }
            }
        }

        /**
         * change dx based on user input
         */
        protected override function Walk(elapsedTime:Number):void
        {
            var desiredDirection:Number = 0;
            if (InputUtil.IsDown(Keyboard.RIGHT))
            {
                ++desiredDirection;
            }
            if (InputUtil.IsDown(Keyboard.LEFT))
            {
                --desiredDirection;
            }

            var currentDirection:Number = 0;
            if (this.dx > 0)
                currentDirection = 1;
            else if (this.dx < 0)
                currentDirection = -1;

            // controls a bit differently on the ground
            if (IsOnGround)
            {
                // changing direction or stopping
                if ( (currentDirection != 0) && (desiredDirection != currentDirection) )
                {
                    var ddx:Number = Constants.PLAYER_STOP_ACCEL * elapsedTime;

                    if ( ddx > Math.abs(this.dx) )
                    {
                        this.dx = 0;
                    }
                    else
                    {
                        if (this.dx > 0 )
                            this.dx -= ddx;
                        else
                            this.dx += ddx;
                    }
                }
                else // accelerating
                {
                    this.dx += desiredDirection * Constants.PLAYER_WALK_ACCEL * elapsedTime;
                }
            }
            else
            {
                this.dx += desiredDirection * Constants.PLAYER_AIR_ACCEL * elapsedTime;
            }

            // max speed
            if (this.dx > Constants.PLAYER_WALK_MAX_SPEED)
            {
                this.dx = Constants.PLAYER_WALK_MAX_SPEED;
            }
            if (this.dx < -Constants.PLAYER_WALK_MAX_SPEED)
            {
                this.dx = -Constants.PLAYER_WALK_MAX_SPEED;
            }

            // flip graphics
            if (desiredDirection > 0)
                this.scaleX = 1.0;
            if (desiredDirection < 0)
                this.scaleX = -1.0;
        }

        /**
         *
         */
        protected override  function Jumping(elapsedTime:Number):void
        {
            if ( InputUtil.IsUp(mJumpKey) || IsOnGround)
            {
                mIsJumping = false;
                mJumpKey = 0;
            }

            // if we aren't holding the jump button anymore, cut
            // off the jump height
            if (!IsOnGround && !mIsJumping && (this.dy < 0) )
            {
                this.dy += Constants.PLAYER_JUMP_STOP_ACCEL * elapsedTime;

                // don't shoot us down, though
                if (this.dy > 0)
                    this.dy = 0;
            }

            // starting a new jump
            var justPressedJumpKey:uint = 0;
            for each (var keyCode:uint in JUMP_KEYS)
            {
                if (InputUtil.WasJustPressed(keyCode))
                {
                    justPressedJumpKey = keyCode;
                }
            }

            if (InputUtil.WasJustPressed(justPressedJumpKey))
            {
                if (IsOnGround || (mAirJumpsRemaining > 0) )
                {
                    this.dy = -Constants.PLAYER_JUMP_SPEED;
                    mIsJumping = true;
                    mJumpKey = justPressedJumpKey;
                }
                if (!IsOnGround)
                {
                    --mAirJumpsRemaining;
                }
            }

            // reset air jumps after landing
            if (IsOnGround)
            {
                mAirJumpsRemaining = Constants.NUM_AIR_JUMPS;
            }
        }

        /**
         *
         */
        protected override function Restart():void
        {
            super.Restart();
            mIsJumping = false;
            mAirJumpsRemaining = Constants.NUM_AIR_JUMPS;
        }

        /**
         * Add restart check to bas update
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            if (!IsAlive())
            {
                Main.sInstance.SwitchToScript(new Intro());
            }
        }
    }

}
