﻿package meyermike.minild9 
{
    import flash.display.Bitmap;
    import meyermike.GameObject;
    
    /**
     * ...
     * @author Mike Meyer
     */
    public class Enemy extends PlatformerCharacter
    {
        [Embed(source='../../../assets/enemy.png')]
        private static var GraphicClass:Class;
        
        private static const ANIM_FRAME_LENGTH:Number = 0.15;
        private static const WALK_SPEED:Number = 20;
        private static const HOP_SPEED:Number = 20;
        private static const STOMPED_YSCALE:Number = 0.3;
        
        private var mAnimTimer:Number;
        private var mGraphic:Bitmap;
        
        private var mDirection:Number;
        private var mIsStomped:Boolean;
        
        /**
         * initial setup and re-init when reusing
         */
        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            Restart();
        }
        
        
        /**
         * 
         */
        private function CreateGraphics():void
        {
            if (null == mGraphic)
            {
                mGraphic = new GraphicClass();
                mGraphic.x = -0.5 * mGraphic.width;
                mGraphic.y = -0.5 * mGraphic.height;
                this.addChild(mGraphic);
            }
        }
        
        /**
         * animations and per-frame behavior
         */
        public override function Update(elapsedTime:Number):void 
        {
            super.Update(elapsedTime);
            
            if (!mIsStomped)
            {
                // walk
                this.dx = mDirection * WALK_SPEED;
                
                // animate
                mAnimTimer -= elapsedTime;
                if (mAnimTimer <= 0.0)
                {
                    this.scaleX *= -1;
                    mAnimTimer = ANIM_FRAME_LENGTH;
                }
            }
        }
        
        /**
         * 
         */
        public function StartWalkingRight():void
        {
            mDirection = 1;
            
            // hop a bit because it looks better
            this.dy = -HOP_SPEED;
        }
        
        /**
         * 
         */
        public function StartWalkingLeft():void
        {
            mDirection = -1;
            
            // hop a bit because it looks better
            this.dy = -HOP_SPEED;
        }
         
        /**
         * set initial values after spawning
         */
        protected override function Restart():void
        {
            super.Restart();
            mAnimTimer = ANIM_FRAME_LENGTH;
            mDirection = 0;
            mIsStomped = false;
            this.scaleY = 1.0;
        }
        
        /**
         * check whether this enemy has been stomped
         */
        public function IsStomped():Boolean
        {
            return mIsStomped;
        }
        
        /**
         * Stomp the enemy
         */
        public function Stomp():void
        {
            mIsStomped = true;
            this.dx = 0;
            this.scaleY = STOMPED_YSCALE;
        }
    }
    
}