﻿package meyermike.minild9 
{
    import flash.display.Bitmap;
    import flash.events.Event;
    import meyermike.collision.CollisionInfo;
    
    /**
     * ...
     * @author Mike Meyer
     */
    public class Person extends PlatformerCharacter
    {
        [Embed(source='../../../assets/person_flailing.png')]
        private static var FlailGraphicClass:Class;
        
        [Embed(source='../../../assets/person_rescued.png')]
        private static var RescuedGraphicClass:Class;
        
        protected static const LETHAL_FALL_SPEED:Number = PlatformerCharacter.MAX_FALL_SPEED;
        
        private static const FLAIL_ANIM_FRAME_LENGTH:Number = 0.15;
        private var mAnimTimer:Number;
        
        private var mFlailGraphic:Bitmap;
        private var mRescuedGraphic:Bitmap;
        
        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            Restart();
        }
        
        /**
         * 
         */
        private function CreateGraphics():void
        {
            if (null == mFlailGraphic)
            {
                mFlailGraphic = new FlailGraphicClass();
                mFlailGraphic.x = -0.5 * mFlailGraphic.width;
                mFlailGraphic.y = -0.5 * mFlailGraphic.height;
                this.addChild(mFlailGraphic);
            }
            
            if (null == mRescuedGraphic)
            {
                mRescuedGraphic = new RescuedGraphicClass();
                mRescuedGraphic.x = -0.5 * mRescuedGraphic.width;
                mRescuedGraphic.y = -0.5 * mRescuedGraphic.height;
                this.addChild(mRescuedGraphic);
            }
        }
        
        /**
         * 
         */
        public override function Update(elapsedTime:Number):void 
        {
            super.Update(elapsedTime);
            
            mAnimTimer -= elapsedTime;
            if (mAnimTimer <= 0.0)
            {
                this.scaleX *= -1;
                mAnimTimer = FLAIL_ANIM_FRAME_LENGTH;
            }
        }
         
        /**
         * set initial values after spawning
         */
        protected override function Restart():void
        {
            super.Restart();
            
            mAnimTimer = FLAIL_ANIM_FRAME_LENGTH;
            StartFlailing();
            this.dr = 0.0;
            this.rotation = 0;
        }
        
        /**
         * switch to the flail wildly animation
         */
        public function StartFlailing():void
        {
            mFlailGraphic.visible = true;
            mRescuedGraphic.visible = false;
        }
        
        /**
         * switch to a calmer animation
         */
        public function Rescue():void
        {
            mFlailGraphic.visible = false;
            mRescuedGraphic.visible = true;
            
            // stop him.  mostly so you can save them from falls.
            this.dx = 0;
            this.dy = 0;
        }
        
        /**
         * Start doing the dying animation
         */
        public override function StartDeath():void
        {
            super.StartDeath();
            StartFlailing();
        }
        
        
        /**
         * these guys need to die when they fall too far
         */
        public override function CollisionOccurred(ci:CollisionInfo, object:Object):void
        {
            var lethalFall:Boolean = (ci.normal.y < 0) && (this.dy >= Person.LETHAL_FALL_SPEED);
            super.CollisionOccurred(ci, object);
            
            if ( lethalFall )
            {
                this.StartDeath();
            }
        }
    }
    
}