﻿package meyermike.minild9 
{
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.LineScaleMode;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import flash.ui.Keyboard;
    import meyermike.InputUtil;
    import meyermike.Main;
    
    /**
     * ...
     * @author Mike Meyer
     */
    public class Player extends PlatformerCharacter
    {
        [Embed(source='../../../assets/player_idle.png')]
        private static var IdleGraphic:Class;
        
        [Embed(source='../../../assets/player_jump.png')]
        private static var JumpGraphic:Class;
        
        private static const PLAYER_WIDTH:Number = 10;
        private static const PLAYER_HEIGHT:Number = 20;
        
        private var mIdle:Bitmap;
        private var mCape:Cape;
        private var mRun:PlayerRunAnim;
        private var mJump:Bitmap;
        
        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            Restart();
        }
        
        /**
         * Reset the player to his starting position
         */
        protected override function Restart():void
        {
            super.Restart();
            
            this.dr = 0.0;
            this.rotation = 0;
            
            SwitchToIdleAnim();
        }
        
        private function CreateGraphics():void
        {
            if (null == mCape)
            {
                mCape = new Cape();
                mCape.scaleX = 2;
                mCape.scaleY = 2;
                this.addChild(mCape);
            }
            if (null == mRun)
            {
                mRun = new PlayerRunAnim();
                this.addChild(mRun);
            }
            
            if (null == mIdle)
            {
                mIdle = new IdleGraphic();
                mIdle.x = -0.5 * mIdle.width;
                mIdle.y = -0.5 * mIdle.height;
                this.addChild(mIdle);
            }
            
            if (null == mJump)
            {
                mJump = new JumpGraphic();
                mJump.x = -0.5 * mJump.width;
                mJump.y = -0.5 * mJump.height;
                this.addChild(mJump);
            }
        }
        
        /**
         * per-frame update
         * @param	elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);
            
            // animations
            mCape.Update(elapsedTime);
            mRun.Update(elapsedTime);
            
            // walking
            this.dx = 0;
            if (InputUtil.IsDown(Keyboard.RIGHT))
            {
                this.dx += Constants.PLAYER_WALK_SPEED;
                
                this.scaleX = 1;
            }
            if (InputUtil.IsDown(Keyboard.LEFT))
            {
                this.dx -= Constants.PLAYER_WALK_SPEED;
                
                this.scaleX = -1;
            }
            
            // jumping
            if (InputUtil.WasJustPressed(Keyboard.UP))
            {
                if (IsOnGround)
                {
                    this.dy = -Constants.PLAYER_JUMP_SPEED;
                }
            }
            if (InputUtil.WasJustReleased(Keyboard.UP))
            {
                if (this.dy < 0)
                {
                    this.dy = 0;
                }
            }
            
            
            // switch to the correct animation
            if (IsOnGround)
            {
                if (this.dx == 0.0)
                {
                    SwitchToIdleAnim();
                }
                else
                {
                    SwitchToRunAnim();
                }
            }
            else
            {
                SwitchToJumpAnim();
            }
        }
        
        /**
         * clean up
         */
        public override function Shutdown():void 
        {
            super.Shutdown();
        }
        
        /**
         * Get the collision bounds for collision testing
         */
        public override function GetCollisionBounds():Rectangle
        {
            var transformed:Point = localToGlobal(new Point(0, 0));
            return new Rectangle(transformed.x - 5, transformed.y - 10, 10, 20);
        }
        
        /**
         * 
         */
        private function SwitchToIdleAnim():void
        {
            mIdle.visible = true;
            mRun.visible = false;
            mJump.visible = false;
        }
        
        /**
         * 
         */
        private function SwitchToRunAnim():void
        {
            mIdle.visible = false;
            mRun.visible = true;
            mJump.visible = false;
        }
        
        /**
         * 
         */
        private function SwitchToJumpAnim():void
        {
            mIdle.visible = false;
            mRun.visible = false;
            mJump.visible = true;
        }
    }
    
}