﻿package meyermike.widget
{
    import flash.display.Sprite;
    import flash.display.DisplayObjectContainer;
    import meyermike.Main;
    import meyermike.squareversus.Constants;
    import meyermike.squareversus.GameObject;
    import meyermike.squareversus.script.Script;
    import meyermike.Util;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingText extends GameObject
    {
        protected var mPulseHolder:DisplayObjectContainer;
        protected var mPulseTextField:TextField;
        protected var mTextField:TextField;

        protected var mColor:Number = Constants.DESAT_YELLOW;

        private static const PULSE_MAX_SCALE:Number = 1.25;
        private static const PULSE_MAX_ALPHA:Number = 0.375;
        private static const PULSE_MIN_ALPHA:Number = 0.0;
        private var mPulseTimer:Number;

        protected var mWidth:Number;
        protected var mHeight:Number;

        public function PulsingText(config:Object = null)
        {
            super(config);

            ResetDefaults();
        }

        public override function Init(config:Object = null):void
        {
            super.Init(config);

            CreateGraphics();

            // make sure color got set
            if ( (null == config)
               ||(null == config.textColor) )
            {
                mPulseTextField.SetFontColor(mColor);
                mTextField.SetFontColor(mColor);
            }
        }

        protected function CreateGraphics():void
        {
            if (null == mPulseHolder)
            {
                mPulseHolder = new Sprite();
                mPulseHolder.x = mWidth * 0.5;
                mPulseHolder.y = mHeight * 0.5;
                this.addChild(mPulseHolder);
            }

            if (null == mPulseTextField)
            {
                mPulseTextField = new TextField();
                mPulseTextField.CenterOn(0, 0);
                mPulseHolder.addChild(mPulseTextField);
            }

            if (null == mTextField)
            {
                mTextField = new TextField();
                mTextField.CenterOn(mWidth*0.5, mHeight*0.5);
                this.addChild(mTextField);
            }
        }

        public override function Beat(beat:Number):void
        {
            super.Beat(beat);

            mPulseTimer = 0;
        }

        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);
            
            var pulseDuration:Number = Main.sInstance.GetScript().GetSecondsPerBeat() * 0.5;

            mPulseTimer += elapsedTime;

            var t:Number = Math.min(1.0, mPulseTimer / PULSE_DURATION);
            mPulseHolder.alpha = Util.Lerp(PULSE_MAX_ALPHA, PULSE_MIN_ALPHA, t);
            mPulseHolder.scaleX = Util.Lerp( 1.0, PULSE_MAX_SCALE, t);
            mPulseHolder.scaleY = mPulseHolder.scaleX;
        }

        /**
         * center the textfield on some point.  CALL SET TEXT *FIRST*
         */
        public function CenterOn(x:Number, y:Number):void
        {
            this.x = x - (mWidth * 0.5);
            this.y = y - (mHeight * 0.5);
        }
        
        /**
         * Change the displayed text
         */
        public function SetText(text:String):void
        {
            mPulseTextField.SetText(text);
            mPulseTextField.CenterOn(0,0);
            
            mTextField.SetText(text);
            mTextField.CenterOn(mWidth*0.5, mHeight*0.5);
        }
        
        /**
         * Change the displayed font size.
         */
        public function SetFontSize(desiredSize:Number):void
        {
            mPulseTextField.SetFontSize(desiredSize);
            mPulseTextField.CenterOn(0,0);
            
            mTextField.SetFontSize(desiredSize);
            mTextField.CenterOn(mWidth*0.5, mHeight*0.5);
        }
        
        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        public override function Shutdown():void
        {
            ResetDefaults();
            super.Shutdown();
        }
        
        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        private function ResetDefaults():void
        {
            mPulseTimer = PULSE_DURATION;
            mWidth = 500;
            mHeight = 80;
            
            if (null != mPulseTextField)
            {
                mPulseTextField.ResetDefaults();
            }
            
            if (null != mTextField)
            {
                mTextField.ResetDefaults();
            }
        }
    }

}