/**
 * Wrapper for Flash text fields to add convenient methods and
 * automatically set good defaults for this game.
 */
package meyermike.widget
{
    import flash.display.Sprite;
    import flash.text.TextField;
    import flash.text.TextFieldAutoSize;
    import flash.text.TextFieldType;
    import flash.text.TextFormat;
    import meyermike.GameObject;
    import meyermike.constants.Colors;

    public class TextField extends GameObject
    {
        protected var DEFAULT_FONT_SIZE:Number = 72;
        protected var DEFAULT_FONT:String = "DefaultTextFieldFont";
        protected var DEFAULT_FONT_COLOR:Number = Colors.BLACK;

        [Embed(source = '../../../assets/Fonts/Gputeks-Bold.ttf', fontFamily='DefaultTextFieldFont', fontWeight='bold')]
        private static var Font:Class;

        /**
         * actual textfield to wrap
         */
        protected var mTextField:flash.text.TextField;

        public function TextField():void
        {
            super();

            if (null == mTextField)
            {
                mTextField = new flash.text.TextField();
                this.addChild(mTextField);
            }
            ResetDefaults();
        }

        /**
         * this function called for both initial setup and to re-init for reuse
         */
        public override function Init():void
        {
            super.Init();
            ResetDefaults();
        }

        /**
         * Change the displayed text.  WILL SCREW UP ALIGNMENT.
         */
        public function SetText(desiredText:String):void
        {
            mTextField.text = desiredText;
            ResetSize();
        }

        /**
         * Get the displayed text.
         */
        public function GetText():String
        {
            return mTextField.text;
        }

        /**
         * Change the font.  WILL SCREW UP ALIGNMENT.
         */
        public function SetFont(font:String):void
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            fmt.font = font;
            mTextField.defaultTextFormat = fmt;
            mTextField.setTextFormat(fmt);

            ResetSize();
        }

        /**
         * Change the displayed font size.  WILL SCREW UP ALIGNMENT.
         */
        public function SetFontSize(desiredSize:Number):void
        {
            var fmt:TextFormat = mTextField.getTextFormat();
            fmt.size = desiredSize;
            mTextField.defaultTextFormat = fmt;
            mTextField.setTextFormat(fmt);

            ResetSize();
        }

        /**
         * used internally to keep .width and .height sensible
         */
        protected function ResetSize():void
        {
            mTextField.width = mTextField.textWidth + /*horrid fudge factor*/ 3 ;
            //mTextField.height = mTextField.textHeight; // setting height to textheight was pretty wrong...
        }

        /**
         * center the textfield on some point.  CALL SETTEXT() *FIRST*
         */
        public function CenterOn(center_x:Number, center_y:Number):void
        {
            this.x = center_x - (mTextField.textWidth * 0.5);
            this.y = center_y - (mTextField.textHeight * 0.5);
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        public function ResetDefaults():void
        {
            this.x = 0;
            this.y = 0;
            mTextField.selectable = false;
            mTextField.embedFonts = true;

            var format:TextFormat = new TextFormat();
            format.font = this.DEFAULT_FONT;
            format.size = this.DEFAULT_FONT_SIZE;
            mTextField.defaultTextFormat = format;
            mTextField.setTextFormat(format);
            mTextField.textColor = this.DEFAULT_FONT_COLOR;
            mTextField.text = "...";

            ResetSize();
        }

        /**
         * Change the color of the displayed text
         */
        public function SetFontColor(color:Number):void
        {
            mTextField.textColor = color;
        }
    }
}
